import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.SortedSet;


public class StableMarriageGenerator {


	public static void main(String[] args) {


		HashMap<String, Integer> args_map = readArgs(args);

		if(args_map.containsKey("-help")  || args_map.containsKey("--help") || args_map.containsKey("-h"))
		{
			printHelp();
			return;
		}

		if(!args_map.containsKey("-npers") && !args_map.containsKey("-maxpers") && !args_map.containsKey("-n"))
		{
			printError("The number of men/women must be specified");
			return;
		}

		//		int nPerson = Integer.parseInt(args[0]);
		//		int percClasses = Integer.parseInt(args[1]);
		//		
		//		
		int personMax = args_map.get("-n");

		int personMin = ( args_map.get("-minpers") != null ? args_map.get("-minpers") : args_map.get("-n")); 
		args_map.put("-minpers", personMin);

		int personStep = ( args_map.get("-steppers") != null ? args_map.get("-steppers") : 1);
		args_map.put("-steppers", personStep);

		int percMax = ( args_map.get("-p") != null ? args_map.get("-p") : 25);
		args_map.put("-p", percMax);

		int percMin = ( args_map.get("-minperc") != null ? args_map.get("-minperc") : percMax);
		args_map.put("-minperc", percMin);

		int percStep = ( args_map.get("-stepperc") != null ? args_map.get("-stepperc") : 1);
		args_map.put("-stepperc", percStep);

		int perSize = ( args_map.get("-i") != null ? args_map.get("-i") : 1);

		String nomeFile = ( args_map.get("-stdout") == null ? "StableMarriage" : null);//findFilename(args);

		if(args_map.containsKey("-v") || args_map.containsKey("--verbose"))
			printOptions(args_map);

		try {
			int ni=0;
			for(int i = 0; i <perSize; i++)
				for(int np = personMin; np <=personMax; np+=personStep)
					for(int pp = percMin; pp <=percMax; pp+=percStep)
					{
						PrintWriter p;
						if(nomeFile  == null)
						{
							p = new PrintWriter(System.out);
						}
						else
						{
							String name = (args_map.containsKey("-namesize")? 
									nomeFile+"-n-"+np+"-p-"+pp+"-i-"+i+".asp" :
										""+ni+"-"+nomeFile+"-0-0.asp");
							p= new PrintWriter(new FileWriter(name));
						}

						p.println("%instance number:"+ni);

						generateInstance(np, pp, p);

						p.close();

						ni++;
					}	
		} catch (IOException e) {
			System.err.println("Error writing file:");
		}
	}

	private static String findFilename(String[] args) {
		for(String s : args)
			if(s.charAt(0) != '-')
				return s;
		return null;
	}

	private static void printHelp() {

		System.out.println("Strong Stable Marriage with ties - instance generator - by F.Ricca & M. Manna");
		System.out.println("");
		System.out.println("SYNOPSIS:");
		System.out.println("java Generator -n=MAX_N_OF_PERSONS [options]");
		System.out.println("");
		System.out.println("-n, -maxperson, -npers: (max) number of men/women (*mandatory*)");
		System.out.println("-minperson: min number of men/women");
		System.out.println("-stepperson: step increment for n");
		System.out.println("-p, -maxperc, -nperc: admit at most n*p/100 equivalence classes");
		System.out.println("-minperc: min percentage of equivalent classes");
		System.out.println("-stepperc: step increment for p");
		System.out.println("-i, --instancepersize: number of instances per size");
		System.out.println("-namesize: filename reveals instance size");
		System.out.println("");
		System.out.println("Example: java Generator -n=35 -p=25");
	}

	private static void printOptions(HashMap<String, Integer> args_map) {
		System.err.println("Running generator with options: ");

		ArrayList<String> array = new ArrayList<String>(); 
		array.addAll(args_map.keySet());
		java.util.Collections.sort(array);

		for(String s : array)
		{
			System.err.print(s);
			Integer val = args_map.get(s);
			if(val!=-1)
				System.err.println(" "+val);
			else
				System.err.println("");
		}

	}

	private static void printError(String string) {
		System.err.println(string);
		System.err.println("");
		printHelp();

	}

	private static HashMap<String, Integer> readArgs(String[] args) {
		HashMap<String, Integer> ris = new HashMap<String, Integer>();

		String opt;
		String val; 

		for(String s : args)
		{
			int uguale = s.indexOf('=');
			if(uguale > 0)
			{
				opt = s.substring(0, uguale);
				val = s.substring(uguale+1);
				try{
					if(opt.equals("-npers") || opt.equals("-n") || opt.equals("-maxpers"))
						opt = "-n";
					if(opt.equals("-nperc") || opt.equals("-p") || opt.equals("-maxperc"))
						opt = "-p";
					if(opt.equals("-i") || opt.equals("--instancepersize"))
						opt = "-i";
					ris.put(opt, Integer.parseInt(val));
				}catch (NumberFormatException e) {
					System.err.println("Expected integer parameter, option"+opt+" ignored");
				}
			}
			else
			{
				opt = s;
				val = s;
				ris.put(opt, -1);
			}
		}

		return ris;
	}

	private static void generateInstance(int nPerson, int percClasses, PrintWriter out) {
		int[][] mPef = new int [nPerson][nPerson];
		int[][] wPef = new int [nPerson][nPerson];

		randomTrivial(nPerson, percClasses, mPef, wPef);

		print(mPef,"manAssignsScore", out);
		print(wPef,"womanAssignsScore", out);
	}

	private static void randomTrivial(int nPerson, int percClasses, int[][] mPef, int[][] wPef) {
		for(int m = 0; m < nPerson; m++)
			for(int w = 0; w < nPerson; w++)
			{
				mPef[m][w] = rand(nPerson*percClasses/100);
				wPef[w][m] = rand(nPerson*percClasses/100);
			}
	}

	private static void print(int[][] pef, String pName, PrintWriter out) {

		char p = pName.charAt(0);
		char s = (p == 'm' ? 'w' : 'm');

		for (int i = 1; i <= pef.length; i++) {
			for (int j = 1; j <= pef.length; j++) {
				out.println(pName+"("+p+i+","+s+j+","+pef[i-1][j-1]+").");
			}
		}
	}

	private static int rand(int person) {
		double num = Math.random() * (person - 1);
		return (int) Math.round(num) + 1;
	}

}
